package sun.awt.image;

import java.awt.image.WritableRaster;
import java.awt.image.Raster;
import java.awt.image.RasterFormatException;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.PixelInterleavedSampleModel;
import java.awt.image.DataBufferByte;
import java.awt.image.ComponentSampleModel;
import java.awt.image.DataBuffer;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.image.SampleModel;

public class ByteInterleavedRaster extends ByteComponentRaster
{
    boolean inOrder;
    int dbOffset;
    int dbOffsetPacked;
    boolean packed;
    int[] bitMasks;
    int[] bitOffsets;
    private int maxX;
    private int maxY;
    
    public ByteInterleavedRaster(SampleModel sampleModel, Point point) {
        this(sampleModel, sampleModel.createDataBuffer(), new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public ByteInterleavedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Point point) {
        this(sampleModel, dataBuffer, new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    private boolean isInterleaved(ComponentSampleModel componentSampleModel) {
        int numBands = this.sampleModel.getNumBands();
        if (numBands == 1) {
            return true;
        }
        int[] bankIndices = componentSampleModel.getBankIndices();
        int i = 0;
        while (i < numBands) {
            if (bankIndices[i] != 0) {
                return false;
            }
            ++i;
        }
        int[] bandOffsets = componentSampleModel.getBandOffsets();
        int n = bandOffsets[0];
        for (int j = 1; j < numBands; ++j) {
            int n2 = bandOffsets[j];
            if (n2 < n) {
                n = n2;
            }
            if (n2 > n) {
                n = n2;
            }
        }
        return n - n < componentSampleModel.getPixelStride();
    }
    
    public ByteInterleavedRaster(SampleModel obj, DataBuffer dataBuffer, Rectangle rectangle, Point point, ByteInterleavedRaster byteInterleavedRaster) {
        super(obj, dataBuffer, rectangle, point, byteInterleavedRaster);
        this.packed = false;
        this.maxX = this.minX + this.width;
        this.maxY = this.minY + this.height;
        if (!(dataBuffer instanceof DataBufferByte)) {
            throw new RasterFormatException("ByteInterleavedRasters must have byte DataBuffers");
        }
        DataBufferByte dataBufferByte = (DataBufferByte)dataBuffer;
        this.data = SunWritableRaster.stealData(dataBufferByte, 0);
        int n = rectangle.x - point.x;
        int n2 = rectangle.y - point.y;
        if (obj instanceof PixelInterleavedSampleModel || (obj instanceof ComponentSampleModel && this.isInterleaved((ComponentSampleModel)obj))) {
            ComponentSampleModel componentSampleModel = (ComponentSampleModel)obj;
            this.scanlineStride = componentSampleModel.getScanlineStride();
            this.pixelStride = componentSampleModel.getPixelStride();
            this.dataOffsets = componentSampleModel.getBandOffsets();
            for (int i = 0; i < this.getNumDataElements(); ++i) {
                int[] dataOffsets2 = this.dataOffsets;
                int n4 = i;
                dataOffsets2[n4] += n * this.pixelStride + n2 * this.scanlineStride;
            }
        }
        else if ((obj instanceof SinglePixelPackedSampleModel)) {
            SinglePixelPackedSampleModel singlePixelPackedSampleModel = (SinglePixelPackedSampleModel)obj;
            this.packed = true;
            this.bitMasks = singlePixelPackedSampleModel.getBitMasks();
            this.bitOffsets = singlePixelPackedSampleModel.getBitOffsets();
            this.scanlineStride = singlePixelPackedSampleModel.getScanlineStride();
            this.pixelStride = 1;
            (this.dataOffsets = new int[1])[0] = dataBufferByte.getOffset();
            int[] dataOffsets = this.dataOffsets;
            int n3 = 0;
            dataOffsets[n3] += n * this.pixelStride + n2 * this.scanlineStride;
        }
        else {
            throw new RasterFormatException("ByteInterleavedRasters must have PixelInterleavedSampleModel, SinglePixelPackedSampleModel or interleaved ComponentSampleModel.  Sample model is " + obj);
        }
        this.bandOffset = this.dataOffsets[0];
        this.dbOffsetPacked = dataBuffer.getOffset() - this.sampleModelTranslateY * this.scanlineStride - this.sampleModelTranslateX * this.pixelStride;
        this.dbOffset = this.dbOffsetPacked - (n * this.pixelStride + n2 * this.scanlineStride);
        this.inOrder = false;
        if (this.numDataElements == this.pixelStride) {
            this.inOrder = true;
            int j = 1;
            while (j < this.numDataElements) {
                if (this.dataOffsets[j] - this.dataOffsets[0] != j) {
                    this.inOrder = false;
                    break;
                }
                ++j;
            }
        }
        this.verify();
        return;
    }
    
    @Override
    public int[] getDataOffsets() {
        return (int[])(int[])this.dataOffsets.clone();
    }
    
    @Override
    public int getDataOffset(int n) {
        return this.dataOffsets[n];
    }
    
    @Override
    public int getScanlineStride() {
        return this.scanlineStride;
    }
    
    @Override
    public int getPixelStride() {
        return this.pixelStride;
    }
    
    @Override
    public byte[] getDataStorage() {
        return this.data;
    }
    
    @Override
    public Object getDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array;
        if (o == null) {
            array = new byte[this.numDataElements];
        }
        else {
            array = (byte[])(byte[])o;
        }
        int n3 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        for (int i = 0; i < this.numDataElements; ++i) {
            array[i] = this.data[this.dataOffsets[i] + n3];
        }
        return array;
    }
    
    @Override
    public Object getDataElements(int n, int n2, int n3, int n4, Object o) {
        return this.getByteData(n, n2, n3, n4, (byte[])(byte[])o);
    }
    
    @Override
    public byte[] getByteData(int n, int n2, int n3, int n4, int n5, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new byte[n3 * n4];
        }
        int n6 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride + this.dataOffsets[n5];
        int n7 = 0;
        if (this.pixelStride == 1) {
            if (this.scanlineStride == n3) {
                System.arraycopy(this.data, n6, array, 0, n3 * n4);
            }
            else {
                for (int k = 0; k < n4; n7 += n3, ++k, n6 += this.scanlineStride) {
                    System.arraycopy(this.data, n6, array, n7, n3);
                }
            }
        }
        else {
            for (int i = 0; i < n4; ++i, n6 += this.scanlineStride) {
                for (int j = 0; j < n3; ++j, n6 += this.pixelStride) {
                    array[n7++] = this.data[n6];
                }
            }
        }
        return array;
    }
    
    @Override
    public byte[] getByteData(int n, int n2, int n3, int n4, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new byte[this.numDataElements * n3 * n4];
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        int n6 = 0;
        if (this.inOrder) {
            int n24 = n5 + this.dataOffsets[0];
            int n25 = n3 * this.pixelStride;
            if (this.scanlineStride == n25) {
                System.arraycopy(this.data, n24, array, n6, n25 * n4);
            }
            else {
                for (int n26 = 0; n26 < n4; n6 += n25, ++n26, n24 += this.scanlineStride) {
                    System.arraycopy(this.data, n24, array, n6, n25);
                }
            }
        }
        else if (this.numDataElements == 1) {
            for (int n21 = n5 + this.dataOffsets[0], n22 = 0; n22 < n4; ++n22, n21 += this.scanlineStride) {
                for (int n21 = n21, n23 = 0; n23 < n3; ++n23, n21 += this.pixelStride) {
                    array[n6++] = this.data[n21];
                }
            }
        }
        else if (this.numDataElements == 2) {
            int n17 = n5 + this.dataOffsets[0];
            int n18 = this.dataOffsets[1] - this.dataOffsets[0];
            for (int n19 = 0; n19 < n4; ++n19, n17 += this.scanlineStride) {
                for (int n20 = 0; n20 < n3; ++n20, n17 += this.pixelStride) {
                    array[n6++] = this.data[n17];
                    array[n6++] = this.data[n17 + n18];
                }
            }
        }
        else if (this.numDataElements == 3) {
            int n12 = n5 + this.dataOffsets[0];
            int n13 = this.dataOffsets[1] - this.dataOffsets[0];
            int n14 = this.dataOffsets[2] - this.dataOffsets[0];
            for (int n15 = 0; n15 < n4; ++n15, n12 += this.scanlineStride) {
                for (int n16 = 0; n16 < n3; ++n16, n12 += this.pixelStride) {
                    array[n6++] = this.data[n12];
                    array[n6++] = this.data[n12 + n13];
                    array[n6++] = this.data[n12 + n14];
                }
            }
        }
        else if (this.numDataElements == 4) {
            int n7 = n5 + this.dataOffsets[0];
            int n8 = this.dataOffsets[1] - this.dataOffsets[0];
            int n9 = this.dataOffsets[2] - this.dataOffsets[0];
            int n10 = this.dataOffsets[3] - this.dataOffsets[0];
            for (int l = 0; l < n4; ++l, n7 += this.scanlineStride) {
                for (int n11 = 0; n11 < n3; ++n11, n7 += this.pixelStride) {
                    array[n6++] = this.data[n7];
                    array[n6++] = this.data[n7 + n8];
                    array[n6++] = this.data[n7 + n9];
                    array[n6++] = this.data[n7 + n10];
                }
            }
        }
        else {
            for (int i = 0; i < n4; ++i, n5 += this.scanlineStride) {
                for (int j = 0; j < n3; ++j, n5 += this.pixelStride) {
                    for (int k = 0; k < this.numDataElements; ++k) {
                        array[n6++] = this.data[this.dataOffsets[k] + n5];
                    }
                }
            }
        }
        return array;
    }
    
    @Override
    public void setDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array = (byte[])(byte[])o;
        int n3 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        for (int i = 0; i < this.numDataElements; ++i) {
            this.data[this.dataOffsets[i] + n3] = array[i];
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, Raster raster) {
        int minX = raster.getMinX();
        int minY = raster.getMinY();
        int n3 = n + minX;
        int n4 = n2 + minY;
        int width = raster.getWidth();
        int height = raster.getHeight();
        if (n3 < this.minX || n4 < this.minY || n3 + width > this.maxX || n4 + height > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        this.setDataElements(n3, n4, minX, minY, width, height, raster);
        return;
    }
    
    private void setDataElements(int n, int n2, int n3, int n4, int w, int n5, Raster raster) {
        if (w <= 0 || n5 <= 0) {
            return;
        }
        int minX = raster.getMinX();
        int minY = raster.getMinY();
        Object dataElements = null;
        if ((raster instanceof ByteInterleavedRaster)) {
            ByteInterleavedRaster byteInterleavedRaster = (ByteInterleavedRaster)raster;
            byte[] dataStorage = byteInterleavedRaster.getDataStorage();
            if (this.inOrder && byteInterleavedRaster.inOrder && this.pixelStride == byteInterleavedRaster.pixelStride) {
                int dataOffset = byteInterleavedRaster.getDataOffset(0);
                int scanlineStride = byteInterleavedRaster.getScanlineStride();
                int n6 = dataOffset + (n4 - minY) * scanlineStride + (n3 - minX) * byteInterleavedRaster.getPixelStride();
                int n7 = this.dataOffsets[0] + (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
                int n8 = w * this.pixelStride;
                for (int i = 0; i < n5; n6 += scanlineStride, n7 += this.scanlineStride, ++i) {
                    System.arraycopy(dataStorage, n6, this.data, n7, n8);
                }
                this.markDirty();
                return;
            }
        }
        for (int j = 0; j < n5; ++j) {
            dataElements = raster.getDataElements(minX, minY + j, w, 1, dataElements);
            this.setDataElements(n, n2 + j, w, 1, dataElements);
        }
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, int n3, int n4, Object o) {
        this.putByteData(n, n2, n3, n4, (byte[])(byte[])o);
    }
    
    @Override
    public void putByteData(int n, int n2, int n3, int n4, int n5, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n6 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride + this.dataOffsets[n5];
        int n7 = 0;
        if (this.pixelStride == 1) {
            if (this.scanlineStride == n3) {
                System.arraycopy(array, 0, this.data, n6, n3 * n4);
            }
            else {
                for (int k = 0; k < n4; n7 += n3, ++k, n6 += this.scanlineStride) {
                    System.arraycopy(array, n7, this.data, n6, n3);
                }
            }
        }
        else {
            for (int i = 0; i < n4; ++i, n6 += this.scanlineStride) {
                for (int j = 0; j < n3; ++j, n6 += this.pixelStride) {
                    this.data[n6] = array[n7++];
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void putByteData(int n, int n2, int n3, int n4, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        int n6 = 0;
        if (this.inOrder) {
            int n24 = n5 + this.dataOffsets[0];
            int n25 = n3 * this.pixelStride;
            if (n25 == this.scanlineStride) {
                System.arraycopy(array, 0, this.data, n24, n25 * n4);
            }
            else {
                for (int n26 = 0; n26 < n4; n6 += n25, ++n26, n24 += this.scanlineStride) {
                    System.arraycopy(array, n6, this.data, n24, n25);
                }
            }
        }
        else if (this.numDataElements == 1) {
            for (int n21 = n5 + this.dataOffsets[0], n22 = 0; n22 < n4; ++n22, n21 += this.scanlineStride) {
                for (int n21 = n21, n23 = 0; n23 < n3; ++n23, n21 += this.pixelStride) {
                    this.data[n21] = array[n6++];
                }
            }
        }
        else if (this.numDataElements == 2) {
            int n17 = n5 + this.dataOffsets[0];
            int n18 = this.dataOffsets[1] - this.dataOffsets[0];
            for (int n19 = 0; n19 < n4; ++n19, n17 += this.scanlineStride) {
                for (int n20 = 0; n20 < n3; ++n20, n17 += this.pixelStride) {
                    this.data[n17] = array[n6++];
                    this.data[n17 + n18] = array[n6++];
                }
            }
        }
        else if (this.numDataElements == 3) {
            int n12 = n5 + this.dataOffsets[0];
            int n13 = this.dataOffsets[1] - this.dataOffsets[0];
            int n14 = this.dataOffsets[2] - this.dataOffsets[0];
            for (int n15 = 0; n15 < n4; ++n15, n12 += this.scanlineStride) {
                for (int n16 = 0; n16 < n3; ++n16, n12 += this.pixelStride) {
                    this.data[n12] = array[n6++];
                    this.data[n12 + n13] = array[n6++];
                    this.data[n12 + n14] = array[n6++];
                }
            }
        }
        else if (this.numDataElements == 4) {
            int n7 = n5 + this.dataOffsets[0];
            int n8 = this.dataOffsets[1] - this.dataOffsets[0];
            int n9 = this.dataOffsets[2] - this.dataOffsets[0];
            int n10 = this.dataOffsets[3] - this.dataOffsets[0];
            for (int l = 0; l < n4; ++l, n7 += this.scanlineStride) {
                for (int n11 = 0; n11 < n3; ++n11, n7 += this.pixelStride) {
                    this.data[n7] = array[n6++];
                    this.data[n7 + n8] = array[n6++];
                    this.data[n7 + n9] = array[n6++];
                    this.data[n7 + n10] = array[n6++];
                }
            }
        }
        else {
            for (int i = 0; i < n4; ++i, n5 += this.scanlineStride) {
                for (int j = 0; j < n3; ++j, n5 += this.pixelStride) {
                    for (int k = 0; k < this.numDataElements; ++k) {
                        this.data[this.dataOffsets[k] + n5] = array[n6++];
                    }
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public int getSample(int n, int n2, int n3) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (this.packed) {
            return (this.data[n2 * this.scanlineStride + n + this.dbOffsetPacked] & this.bitMasks[n3]) >>> this.bitOffsets[n3];
        }
        return this.data[n2 * this.scanlineStride + n * this.pixelStride + this.dbOffset + this.dataOffsets[n3]] & 0xFF;
    }
    
    @Override
    public void setSample(int n, int n2, int n3, int n4) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (this.packed) {
            int n5 = n2 * this.scanlineStride + n + this.dbOffsetPacked;
            int n6 = this.bitMasks[n3];
            this.data[n5] = (byte)((byte)(this.data[n5] & ~n6) | (n4 << this.bitOffsets[n3] & n6));
        }
        else {
            this.data[n2 * this.scanlineStride + n * this.pixelStride + this.dbOffset + this.dataOffsets[n3]] = (byte)n4;
        }
        this.markDirty();
        return;
    }
    
    @Override
    public int[] getSamples(int n, int n2, int n3, int n4, int n5, int[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array != null) {
        }
        else {
            array = new int[n3 * n4];
        }
        int n6 = n2 * this.scanlineStride + n * this.pixelStride;
        int n7 = 0;
        if (this.packed) {
            int n9 = n6 + this.dbOffsetPacked;
            int n10 = this.bitMasks[n5];
            int n11 = this.bitOffsets[n5];
            for (int k = 0; k < n4; n9 += this.scanlineStride, ++k) {
                for (int l = 0; l < n3; ++l) {
                    array[n7++] = (this.data[n9++] & n10) >>> n11;
                }
            }
            return array;
        }
        for (int n8 = n6 + (this.dbOffset + this.dataOffsets[n5]), i = 0; i < n4; n8 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; ++j) {
                array[n7++] = (this.data[n8] & 0xFF);
                n8 += this.pixelStride;
            }
        }
        return array;
    }
    
    @Override
    public void setSamples(int n, int n2, int n3, int n4, int n5, int[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n6 = n2 * this.scanlineStride + n * this.pixelStride;
        int n7 = 0;
        if (this.packed) {
            int n9 = n6 + this.dbOffsetPacked;
            int n10 = this.bitMasks[n5];
            for (int k = 0; k < n4; n9 += this.scanlineStride, ++k) {
                for (int l = 0; l < n3; ++l) {
                    this.data[n9++] = (byte)((byte)(this.data[n9] & ~n10) | (array[n7++] << this.bitOffsets[n5] & n10));
                }
            }
        }
        else {
            for (int n8 = n6 + (this.dbOffset + this.dataOffsets[n5]), i = 0; i < n4; n8 += this.scanlineStride, ++i) {
                for (int j = 0; j < n3; ++j) {
                    this.data[n8] = (byte)array[n7++];
                    n8 += this.pixelStride;
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public int[] getPixels(int n, int n2, int n3, int n4, int[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array != null) {
        }
        else {
            array = new int[n3 * n4 * this.numBands];
        }
        int n5 = n2 * this.scanlineStride + n * this.pixelStride;
        int n6 = 0;
        if (this.packed) {
            for (int n26 = n5 + this.dbOffsetPacked, n27 = 0; n27 < n4; n26 += this.scanlineStride, ++n27) {
                for (int n28 = 0; n28 < n3; ++n28) {
                    int n29 = this.data[n26 + n28];
                    for (int n30 = 0; n30 < this.numBands; ++n30) {
                        array[n6++] = (n29 & this.bitMasks[n30]) >>> this.bitOffsets[n30];
                    }
                }
            }
        }
        else {
            int n7 = n5 + this.dbOffset;
            int n8 = this.dataOffsets[0];
            if (this.numBands == 1) {
                for (int n23 = 0; n23 < n4; n7 += this.scanlineStride, ++n23) {
                    int n24 = n7 + n8;
                    for (int n25 = 0; n25 < n3; ++n25) {
                        array[n6++] = (this.data[n24] & 0xFF);
                        n24 += this.pixelStride;
                    }
                }
            }
            else if (this.numBands == 2) {
                int n19 = this.dataOffsets[1] - n8;
                for (int n20 = 0; n20 < n4; n7 += this.scanlineStride, ++n20) {
                    int n21 = n7 + n8;
                    for (int n22 = 0; n22 < n3; ++n22) {
                        array[n6++] = (this.data[n21] & 0xFF);
                        array[n6++] = (this.data[n21 + n19] & 0xFF);
                        n21 += this.pixelStride;
                    }
                }
            }
            else if (this.numBands == 3) {
                int n14 = this.dataOffsets[1] - n8;
                int n15 = this.dataOffsets[2] - n8;
                for (int n16 = 0; n16 < n4; n7 += this.scanlineStride, ++n16) {
                    int n17 = n7 + n8;
                    for (int n18 = 0; n18 < n3; ++n18) {
                        array[n6++] = (this.data[n17] & 0xFF);
                        array[n6++] = (this.data[n17 + n14] & 0xFF);
                        array[n6++] = (this.data[n17 + n15] & 0xFF);
                        n17 += this.pixelStride;
                    }
                }
            }
            else if (this.numBands == 4) {
                int n9 = this.dataOffsets[1] - n8;
                int n10 = this.dataOffsets[2] - n8;
                int n11 = this.dataOffsets[3] - n8;
                for (int l = 0; l < n4; n7 += this.scanlineStride, ++l) {
                    int n12 = n7 + n8;
                    for (int n13 = 0; n13 < n3; ++n13) {
                        array[n6++] = (this.data[n12] & 0xFF);
                        array[n6++] = (this.data[n12 + n9] & 0xFF);
                        array[n6++] = (this.data[n12 + n10] & 0xFF);
                        array[n6++] = (this.data[n12 + n11] & 0xFF);
                        n12 += this.pixelStride;
                    }
                }
            }
            else {
                for (int i = 0; i < n4; n7 += this.scanlineStride, ++i) {
                    for (int j = 0; j < n3; n7 += this.pixelStride, ++j) {
                        for (int k = 0; k < this.numBands; ++k) {
                            array[n6++] = (this.data[n7 + this.dataOffsets[k]] & 0xFF);
                        }
                    }
                }
            }
        }
        return array;
    }
    
    @Override
    public void setPixels(int n, int n2, int n3, int n4, int[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n5 = n2 * this.scanlineStride + n * this.pixelStride;
        int n6 = 0;
        if (this.packed) {
            for (int n26 = n5 + this.dbOffsetPacked, n27 = 0; n27 < n4; n26 += this.scanlineStride, ++n27) {
                int n29;
                for (int n28 = 0; n28 < n3; this.data[n26 + n28] = (byte)n29, ++n28) {
                    n29 = 0;
                    for (int n30 = 0; n30 < this.numBands; ++n30) {
                        n29 |= (array[n6++] << this.bitOffsets[n30] & this.bitMasks[n30]);
                    }
                }
            }
        }
        else {
            int n7 = n5 + this.dbOffset;
            int n8 = this.dataOffsets[0];
            if (this.numBands == 1) {
                for (int n23 = 0; n23 < n4; n7 += this.scanlineStride, ++n23) {
                    int n24 = n7 + n8;
                    for (int n25 = 0; n25 < n3; ++n25) {
                        this.data[n24] = (byte)array[n6++];
                        n24 += this.pixelStride;
                    }
                }
            }
            else if (this.numBands == 2) {
                int n19 = this.dataOffsets[1] - n8;
                for (int n20 = 0; n20 < n4; n7 += this.scanlineStride, ++n20) {
                    int n21 = n7 + n8;
                    for (int n22 = 0; n22 < n3; ++n22) {
                        this.data[n21] = (byte)array[n6++];
                        this.data[n21 + n19] = (byte)array[n6++];
                        n21 += this.pixelStride;
                    }
                }
            }
            else if (this.numBands == 3) {
                int n14 = this.dataOffsets[1] - n8;
                int n15 = this.dataOffsets[2] - n8;
                for (int n16 = 0; n16 < n4; n7 += this.scanlineStride, ++n16) {
                    int n17 = n7 + n8;
                    for (int n18 = 0; n18 < n3; ++n18) {
                        this.data[n17] = (byte)array[n6++];
                        this.data[n17 + n14] = (byte)array[n6++];
                        this.data[n17 + n15] = (byte)array[n6++];
                        n17 += this.pixelStride;
                    }
                }
            }
            else if (this.numBands == 4) {
                int n9 = this.dataOffsets[1] - n8;
                int n10 = this.dataOffsets[2] - n8;
                int n11 = this.dataOffsets[3] - n8;
                for (int l = 0; l < n4; n7 += this.scanlineStride, ++l) {
                    int n12 = n7 + n8;
                    for (int n13 = 0; n13 < n3; ++n13) {
                        this.data[n12] = (byte)array[n6++];
                        this.data[n12 + n9] = (byte)array[n6++];
                        this.data[n12 + n10] = (byte)array[n6++];
                        this.data[n12 + n11] = (byte)array[n6++];
                        n12 += this.pixelStride;
                    }
                }
            }
            else {
                for (int i = 0; i < n4; n7 += this.scanlineStride, ++i) {
                    for (int j = 0; j < n3; n7 += this.pixelStride, ++j) {
                        for (int k = 0; k < this.numBands; ++k) {
                            this.data[n7 + this.dataOffsets[k]] = (byte)array[n6++];
                        }
                    }
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void setRect(int dx, int dy, Raster srcRaster) {
        if (!(srcRaster instanceof ByteInterleavedRaster)) {
            super.setRect(dx, dy, srcRaster);
            return;
        }
        int width = srcRaster.getWidth();
        int height = srcRaster.getHeight();
        int minX = srcRaster.getMinX();
        int minY = srcRaster.getMinY();
        int minX2 = dx + minX;
        int minY2 = dy + minY;
        if (minX2 < this.minX) {
            int n = this.minX - minX2;
            width -= n;
            minX += n;
            minX2 = this.minX;
        }
        if (minY2 < this.minY) {
            int n2 = this.minY - minY2;
            height -= n2;
            minY += n2;
            minY2 = this.minY;
        }
        if (minX2 + width > this.maxX) {
            width = this.maxX - minX2;
        }
        if (minY2 + height > this.maxY) {
            height = this.maxY - minY2;
        }
        this.setDataElements(minX2, minY2, minX, minY, width, height, srcRaster);
        return;
    }
    
    @Override
    public Raster createChild(int n, int n2, int n3, int n4, int n5, int n6, int[] array) {
        return this.createWritableChild(n, n2, n3, n4, n5, n6, array);
    }
    
    @Override
    public WritableRaster createWritableChild(int n, int n2, int width, int height, int x, int y, int[] array) {
        if (n < this.minX) {
            throw new RasterFormatException("x lies outside the raster");
        }
        if (n2 < this.minY) {
            throw new RasterFormatException("y lies outside the raster");
        }
        if (n + width < n || n + width > this.minX + this.width) {
            throw new RasterFormatException("(x + width) is outside of Raster");
        }
        if (n2 + height < n2 || n2 + height > this.minY + this.height) {
            throw new RasterFormatException("(y + height) is outside of Raster");
        }
        SampleModel sampleModel;
        if (array != null) {
            sampleModel = this.sampleModel.createSubsetSampleModel(array);
        }
        else {
            sampleModel = this.sampleModel;
        }
        return new ByteInterleavedRaster(sampleModel, this.dataBuffer, new Rectangle(x, y, width, height), new Point(this.sampleModelTranslateX + (x - n), this.sampleModelTranslateY + (y - n2)), this);
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        if (n <= 0 || n2 <= 0) {
            throw new RasterFormatException("negative " + ((n <= 0) ? "width" : "height"));
        }
        return new ByteInterleavedRaster(this.sampleModel.createCompatibleSampleModel(n, n2), new Point(0, 0));
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster() {
        return this.createCompatibleWritableRaster(this.width, this.height);
    }
    
    @Override
    public String toString() {
        return new String("ByteInterleavedRaster: width = " + this.width + " height = " + this.height + " #numDataElements " + this.numDataElements + " dataOff[0] = " + this.dataOffsets[0]);
    }
}
